﻿Shader "NPR/ConvolutionMultipass"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
	}
	SubShader
	{
		// No culling or depth
		Cull Off ZWrite Off ZTest Always

		Pass // horizontal pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}
			
			sampler2D _MainTex;
			float4 _MainTex_TexelSize;
			int boxKernelWidth;

			float4 frag(v2f i) : SV_Target{
				float4 col = fixed4(0,0,0,0);
				float weight = 1.0 / (float)boxKernelWidth; // weight is constant
				for (int u = -boxKernelWidth / 2; u <= boxKernelWidth / 2; ++u) { // visit neighbours (horizontal)
					col += weight *	tex2D(_MainTex, i.uv + float2(u*_MainTex_TexelSize.x, 0));
				}
				return col;
			}
			ENDCG
		}

		// The result of the first pass is grabbed and passed
		//     as _MainTex to the next pass
		GrabPass{}

		Pass // vertical pass, after GrabPass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag

			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert(appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}

			sampler2D _GrabTexture;
			float4 _GrabTexture_TexelSize;
			int boxKernelWidth;

			float4 frag(v2f i) : SV_Target{
#if UNITY_UV_STARTS_AT_TOP
				if (_GrabTexture_TexelSize.y < 0)
				i.uv.y = 1 - i.uv.y;
#endif
				float4 col = float4(0,0,0,0);
				float weight = 1.0 / (float)boxKernelWidth; // weight is constant
				for (int v = -boxKernelWidth / 2; v <= boxKernelWidth / 2; ++v) { // visit neighbours (vertical)
					col += weight *	tex2D(_GrabTexture,	i.uv + float2(0, v*_GrabTexture_TexelSize.y));
				}
				return col;
			}
			ENDCG
		}

		/*
		Pass // vertical pass, if there is no GrabPass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag

			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert(appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}

			sampler2D _MainTex;
			float4 _MainTex_TexelSize;
			int boxKernelWidth;

			float4 frag(v2f i) : SV_Target{
#if UNITY_UV_STARTS_AT_TOP
				if (_MainTex_TexelSize.y < 0)
				i.uv.y = 1 - i.uv.y;
#endif
				float4 col = float4(0,0,0,0);
				float weight = 1.0 / (float)boxKernelWidth; // weight is constant
				for (int v = -boxKernelWidth / 2; v <= boxKernelWidth / 2; ++v) { // visit neighbours (vertical)
					col += weight *	tex2D(_MainTex,	i.uv + float2(0, v*_MainTex_TexelSize.y));
				}
				return col;
			}
			ENDCG
		}
		*/
	}
}
